#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import math
import numpy as np
from messages import Twist

DELTA_DISTANCE = 0.12
C_TURNING_SPEED = 5
C_AVOID_SPEED = 10

class HexapodController:
    def __init__(self):
        pass

    def goto(self, goal, odometry, collision):
        """Method to steer the robot towards the goal position given its current
           odometry and collision status
        Args:
            goal: Pose of the robot goal
            odometry: Perceived odometry of the robot
            collision: bool of the robot collision status
        Returns:
            cmd: Twist steering command
        """
        # Zero velocity steering command. If nothing else happens, just stop.
        result_command = Twist()
        return result_command # TODO for AI expert: Delete me.

        # Check if robot collides with an obstacle. If so, abort the mission completely.
        if collision is not None and collision:
            print("\nOuch!")
            return None

        # We need to know robot's current position and robot's current goal waypoint.
        if odometry is not None and goal is not None:

            # Compute the distance to the goal.
            distance = odometry.pose.dist(goal)

            # Check if the robot is close-enough to the goal. (We can never reach it perfectly.)
            if distance > DELTA_DISTANCE:
                # Unpack our two points on the triangle.
                R = odometry.pose.position
                W = goal.position

                # Compute the triangle side lengths.
                dx = W.x - R.x
                dy = W.y - R.y

                # Compute the desired angular heading from the triangle with sides dx, dy.
                desired_heading_radian = 0 # TODO for AI expert: Compute the angular heading using the math.atan2 function.

                # Get current angular heading.
                actual_heading_radian = odometry.pose.orientation.to_Euler()[0]

                # How wrong is the heading.
                heading_error_radian = ((desired_heading_radian - actual_heading_radian + math.pi) % (2 * math.pi)) - math.pi

                # Set the velocity command to navigate the robot.
                result_command.linear.x = distance
                result_command.angular.z = heading_error_radian * C_TURNING_SPEED
            else:
                print("\nCongratulations! You have reached a waypoint.")
                return None

        return result_command

    def goto_reactive(self, goal, odometry, collision, laser_scan):
        """Method to steer the robot towards the goal position while avoiding
           contact with the obstacles given its current odometry, collision
           status and laser scan data
        Args:
            goal: Pose of the robot goal
            odometry: Perceived odometry of the robot
            collision: bool of the robot collision status
            laser_scan: LaserScan data perceived by the robot
        Returns:
            cmd: Twist steering command
        """

        # Zero velocity steering command. If nothing else happens, just stop.
        result_command = Twist()

        # Check if robot collides with an obstacle. If so, abort the mission completely.
        if collision is not None and collision:
            print("\nOuch!")
            return None

        # We need to know robot's current position and robot's current goal waypoint and also the LiDAR data.
        if laser_scan is not None and odometry is not None and goal is not None:

            # Reuse the existing controller.
            cmd = self.goto(goal, odometry, collision)
            if cmd is None:
                return None

            # Look for distance to the closest obstacle to the left and to the right.
            n = len(laser_scan.distances)
            dist_left = [laser_scan.distances[i] for i in range(n)
                                if laser_scan.distances[i] >= laser_scan.range_min
                                and laser_scan.distances[i] <= laser_scan.range_max and (laser_scan.angle_min + i*laser_scan.angle_increment) <= 0]
            dist_right = [laser_scan.distances[i] for i in range(n)
                                 if laser_scan.distances[i] >= laser_scan.range_min
                                 and laser_scan.distances[i] <= laser_scan.range_max and (laser_scan.angle_min + i*laser_scan.angle_increment) > 0]

            # Find the distance to nearest obstace at each side.
            scan_left = 1 if len(dist_left) == 0 else np.min(dist_left)
            scan_right = 1 if len(dist_right) == 0 else np.min(dist_right)

            # Compute the obstacle-avoiding repulisve force.
            repulsive_force = 1/scan_left - 1/scan_right

            # Modify the angular speed.
            angular_speed = cmd.angular.z + repulsive_force * C_AVOID_SPEED
            result_command.linear.x = cmd.linear.x
            result_command.angular.z = angular_speed

        return result_command

